#include <stdio.h>
#include <string>
#include "texception.h"
#include "traffic.h"
#include "codestruct.h"
#include "codegen.h"
#include "fileadmin.h"

int iniparse (void);
int planparse (void);

Traffic traffic;
CodeStruct codeStruct;
CodeGen codeGen;
FileAdmin fileAdmin;
int errorline;

extern FILE *iniin, *planin;

void showHelp ()
{
  cout << "\nUsage tsc -[?hiomt]\n\n";
  cout << "where:\n";
  cout << "\t?\tShow this message\n";
  cout << "\th\tShow this message\n";
  cout << "\ti\tSpecify the input program file. Default model.plan\n";
  cout << "\to\tSpecify the output simulation model file. Default model.ma\n";
  cout << "\tm\tSpecify the macros simulation file. Default model.macros\n";
  cout << "\tt\tSpecify the template generation file. Default tsc.ini\n";
  exit (1);
}


main (int argc, char **argv)
{
  int returnCode;
  string inputName, outputName, macroName, codeName;

  cout << "--------------------------------------------------------------------------\n";
  cout << "Traffic Simulation Compiler - Version 1.0                                 \n";
  cout << "Departamento de Computacion - Facultad de Ciencias Exactas y Naturales    \n";
  cout << "Universidad de Buenos Aires - Argentina                                   \n";
  cout << "--------------------------------------------------------------------------\n";

  try
  {
    // parameter parsing
    while (--argc)
      if (*argv[argc] == '-')
	switch (argv[argc][1])
	  {
	  case 'i':
	    inputName = argv[argc] + 2;
	    break;
	  case 'o':
	    outputName = argv[argc] + 2;
	    break;
	  case 'm':
	    macroName = argv[argc] + 2;
	    break;
	  case 't':
	    codeName = argv[argc] + 2;
	    break;
	  case '?':
	  case 'h':
	    showHelp ();
	  default:
	    cout << "Warning... invalid parameter " << argv[argc] << " !" << endl;
	    showHelp ();
	  }
      else
	{
	  cout << "Warning... invalid parameter " << argv[argc] << " !" << endl;
	  showHelp ();
	}

    if (inputName == "")
      inputName = "model.plan";
    else
      {
	int ubic = inputName.find (".");
	if (ubic == string::npos)
	  inputName += ".plan";
      }

    if (outputName == "")
      {
	outputName = inputName;
	int ubic = outputName.find (".");
	if (ubic == string::npos)
	  outputName += ".ma";
	else
	  outputName = outputName.erase (ubic, outputName.size () - ubic) + ".ma";
      }
    else
      {
	int ubic = outputName.find (".");
	if (ubic == string::npos)
	  outputName += ".ma";
      }

    if (macroName == "")
      {
	macroName = outputName;
	int ubic = macroName.find (".");
	if (ubic == string::npos)
	  macroName += ".macros";
	else
	  macroName = macroName.erase (ubic, macroName.size () - ubic) + ".macros";
      }
    else
      {
	int ubic = macroName.find (".");
	if (ubic == string::npos)
	  macroName += ".macros";
      }

    if (codeName == "")
      codeName = "tsc.ini";
    else
      {
	int ubic = codeName.find (".");
	if (ubic == string::npos)
	  codeName += ".ini";
      }


    planin = fopen (inputName.c_str (), "r");
    if (planin == NULL)
      {
	cout << "Cannot open input file " << inputName << "\n";
	return -1;
      }

    iniin = fopen (codeName.c_str (), "r");
    if (iniin == NULL)
      {
	cout << "Cannot open code file " << codeName << "\n";
	return -1;
      }

    fileAdmin.setFileName (outputName);
    fileAdmin.setMacroFileName (macroName);

    cout << "Parsing     " << inputName << " \n";
    planparse ();

    cout << "Validating  " << inputName << " \n";
    traffic.validate ();

//   traffic.print();

    cout << "Parsing     " << codeName << " \n";
    iniparse ();
//   codeStruct.print();

    cout << "Generating code ...\n";
    codeGen.generate ();

    cout << "Files " << outputName << " and " << macroName << " were successfully constructed !\n";

  }

  catch (TException te)
  {
    te.print ();
  }

}
