/** include files **/
#include "tscgenerator.h"      // base header
#include "message.h"           // class InternalMessage 
#include "parsimu.h"          // class Simulator
#include "distri.h"            // class Distribution 
#include "strutil.h"           // str2Value( ... )
#include "atomicstate.h"
#include "stdlib.h"

/*******************************************************************
* Function Name: TSCGenerator
* Description: constructor
********************************************************************/
TSCGenerator::TSCGenerator( const string &name )
: Atomic( name )
{
	try
	{
		dist = Distribution::create( ParallelMainSimulator::Instance().getParameter( description(), "distribution" ) );
		MASSERT( dist ) ;
		for ( register int i = 0; i < dist->varCount(); i++ )
		{
			string parameter( ParallelMainSimulator::Instance().getParameter( description(), dist->getVar( i ) ) ) ;
			dist->setVar( i, str2Value( parameter ) ) ;
		}

		if( ParallelMainSimulator::Instance().existsParameter( description(), "lanes" ) )
			lanes = str2Int( ParallelMainSimulator::Instance().getParameter( description(), "lanes" ) );
            else 
			lanes = 0;

		if (lanes == 0) {
                MTHROW(MException("TSCGenerator: model " + description() + " has lanes = 0 !" ));
                }

	} catch( InvalidDistribution &e )
	{
		e.addText( "The model " + description() + " has distribution problems!" ) ;
		e.print(cerr);
		MTHROW( e ) ;
	} catch( MException &e )
	{
		MTHROW( e ) ;
	}

	int i;
	string s;
	char buff[5];
	for (i=0; i <lanes; i++){
		//s = "y_t_hayauto";
		//s += itoa(i,buff,10);
		sprintf(buff, "y_t_hayauto%d", i);
                                  s = buff;
		y_t_hayauto.insert(map<int,Port*>::value_type(i,&addOutputPort(s)));
	}

}

/*******************************************************************
* Function Name: initFunction
********************************************************************/
Model &TSCGenerator::initFunction()
{
      lane = 0;
	holdIn( AtomicState ::active, VTime::Zero ) ;
	return *this ;
}

/*******************************************************************
* Function Name: internalFunction
********************************************************************/
Model &TSCGenerator::internalFunction( const InternalMessage & )
{
	holdIn( AtomicState ::active, VTime( static_cast< float >( fabs( distribution().get() ) ) ) ) ;
      lane = lane + 1;
      if (lane == lanes)
         lane = 0;
	return *this ;
}

/*******************************************************************
* Function Name: outputFunction
********************************************************************/
Model &TSCGenerator::outputFunction( const InternalMessage &msg )
{
	sendOutput( msg.time(), *(y_t_hayauto[lane]), 1 ) ;
	return *this ;
}


/*******************************************************************
* Function Name: externalFunction
********************************************************************/
Model &TSCGenerator::externalFunction( const ExternalMessage &msg ){
	throw InvalidMessageException();
}


TSCGenerator::~TSCGenerator()
{
	delete dist;
      delete &y_t_hayauto;
}
